/*
 * Copyright (C) 2004, 2005 Michael Dominic K. (http://www.mdk.org.pl)
 * Copyright (C) 2005 Aurelien Mino <a.mino@free.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using Gtk;
using Gdk;
using Util;
using System;
using System.IO;
using System.Collections;
using System.Text;
using System.Text.RegularExpressions;
using MovieDatabase;

public class Importer : Util.Windows.PleaseWaitWindow {

    private static readonly string ___Msg = "Importing movies from Ant Movie Catalog file.";
   
	static string AMCHeader = " AMC_X.Y Ant Movie Catalog 3.5.x   www.buypin.com    www.antp.be ";

    ArrayList RecordsIndex;
	int AMCMinorVersion, AMCMajorVersion;
    int[] Selection;
	BinaryReader Reader;
	public int ImportedMovies = 0;
   
    /* Constructor and Initializer */
    public Importer(string _filename) : base (___Msg, false) {
        
		Stream __stream;
		try 
		{
			__stream = File.Open(_filename, FileMode.Open);
		}
		catch (Exception excp) 
		{
			throw new Exception ("Specified file doesn't exist or could not be opened for reading!", excp);
		}
		
		Reader = new BinaryReader (__stream, Encoding.GetEncoding(28591));
	}

	public void SetSelection(int[] _selection) 
	{
		Selection = _selection;
	}
   
    /* Actually performs the import operation */
    public override void Start() {
    
        int __RecordsCount = 0;
       
        // TODO: Try/catch + rollbac
		for(int i=0; i < Selection.Length ; i++) {
        
			int __movieNumber = Selection[i];

            Movie __NewMovie = ParseRecord(__movieNumber);
            __NewMovie.TrimSpaces();
            __NewMovie.Data[(int)MovieField.ADDEDDATE] = DateTime.Now.ToFileTime().ToString();
            MovieDB.AddMovie(__NewMovie);
            
            __RecordsCount++;
       }
       
        ImportedMovies =  __RecordsCount;
	}  

    /* Closes file */
    public void CloseFile() {
        if (Reader != null) Reader.Close();
        Reader = null;
    }
    

    /* Destructor */
    ~Importer() {
        CloseFile();
    }
    
	public ArrayList Parse() 
	{
		Reader.BaseStream.Seek(0, SeekOrigin.Begin);

		ArrayList __movies = new ArrayList();
		//ArrayList __covers = new ArrayList();
		RecordsIndex = new ArrayList();
		

		/* Parse and verify the header */
		string __header;
		__header = new string( Reader.ReadChars(AMCHeader.Length) );

		Regex __regex = new Regex(@"AMC_(?<MajorVersion>(\d)).(?<MinorVersion>(\d))");
		MatchCollection __matches = __regex.Matches(__header);

		if (__matches.Count != 0) 
		{
			AMCMajorVersion = int.Parse(__matches[0].Groups["MajorVersion"].Value);
			AMCMinorVersion = int.Parse(__matches[0].Groups["MinorVersion"].Value);
		} 
		else 
		{
			//TODO: Throw a better exception
			throw new Exception ("Specified file is not a valid AMC 3.x file!");
		}

		/* Parse general information */
	
		skipString();						/* Owner's name */
		skipString();						/* Owner's email */

		if ( (AMCMajorVersion <= 3) && (AMCMinorVersion < 5) )
		{
			skipString();					/* Owner's ICQ */
		}

		skipString();						/* Owner's webpage */
		skipString();						/* Database description */
	
		/* For each movie */
		while(Reader.PeekChar() != -1)
		{
			// Saving the position for faster access later
			RecordsIndex.Add( Reader.BaseStream.Position );
			
			__movies.Add(ParseRecord());
				
		}

		return __movies;
	}

	public Movie ParseRecord(int _index) 
	{
		if( _index > RecordsIndex.Count || RecordsIndex[_index] == null) 
		{
			//FIXME: throw an exeption ?
			return null;
		}
		Reader.BaseStream.Seek((long)RecordsIndex[_index], SeekOrigin.Begin);
		
		return ParseRecord();
	}

	public Movie ParseRecord() 
	{
		Movie __movie = new Movie();
		//Pixbuf cover;

		__movie.Data[(int)MovieField.NO]	 = readInt().ToString();									/* ID */
		skipInt();																/* AddDate */
		skipInt();																/* Rating */
			
		if ( (AMCMajorVersion >= 3) && (AMCMinorVersion >= 5) )
		{
			//Rating = new string(Double.Parse(Rating) / 10);
		}

		__movie.Data[(int)MovieField.YEAR] = readInt().ToString();										/* Date */
		__movie.Data[(int)MovieField.TIME] = readInt().ToString();										/* Time */

		skipInt();																/* Video bitrate */
		skipInt();																/* Audio bitrate */

		skipInt();																/* Number of media/files */
		Reader.ReadBoolean();															/* Checked */

		skipString();																/* Media label */
		skipString();																/* Format */
		skipString();																/* Source */
		
		
		//FIXME: we can't import the BORROWER information
		//		 since the BORROWEDDATE is not present in AMC	
		//__movie.Data[(int)MovieField.BORROWER]		= readString();									/* Borrower */
		skipString();
		
		//FIXME: bad solution for the BORROWEDDATE
		//if( __movie.Data[(int)MovieField.BORROWER] != string.Empty )
		//	__movie.Data[(int)MovieField.BORROWEDDATE]	= DateTime.Now;
			
		__movie.Data[(int)MovieField.TITLE]			= readString();									/* Original Title */
		__movie.Data[(int)MovieField.TRANSLATEDTITLE] = readString();										/* Title */

		__movie.Data[(int)MovieField.DIRECTOR]		= readString();										/* Director */
		skipString();																/* Producer */
		__movie.Data[(int)MovieField.COUNTRY]		= readString();										/* Country */
		__movie.Data[(int)MovieField.GENRE]			= readString();									/* Type */
		skipString();																/* Actors */
		skipString();																/* Url */

		__movie.Data[(int)MovieField.DESCRIPTION] 	= readString();										/* Synopsis */
		skipString();																/* Comment */
		__movie.Data[(int)MovieField.VIDEOFORMAT]	= readString();										/* Video format */
		__movie.Data[(int)MovieField.AUDIOFORMAT]	= readString();										/* Audio format */
		skipString();																/* Resolution */
		skipString();																/* Framerate */
			
		skipString();																/* Audio */
		__movie.Data[(int)MovieField.SUBTITLES]		= readString();										/* Subtitle */
		skipString();																/* File size */

		skipString();																/* Picture extension */
		skipString();																/* Picture */


		/*
		//FIXME: Import the cover
		byte[] b;
		b = readBytes();															// Picture		
		if(b.Length != 0) 
		{
			//PixbufLoader pixbufLoader = PixbufLoader.NewWithType("image/jpeg");
			PixbufLoader pixbufLoader = new PixbufLoader();
			pixbufLoader.Write(b);
			pixbufLoader.Close();
			cover = pixbufLoader.Pixbuf;
		}
		else 
		{
			cover = new Pixbuf(new byte[] {},false);
		}
		*/		

		return __movie;
	}

	public uint readInt() 
	{
		uint __i = Reader.ReadUInt32();
		if(__i == UInt32.MaxValue) __i = 0;
		return __i;
	}

	public void skipInt() 
	{
		Reader.BaseStream.Seek(4, SeekOrigin.Current);
	}

	public string readString() 
	{
		int __length = (int)readInt();

		if(__length == 0) 
		{
			return string.Empty;
		}
		else 
		{
			return new string( Reader.ReadChars(__length) );
		}
	}

	public byte[] readBytes() 
	{
		int __length = (int)readInt();

		if(__length == 0) 
		{
			return new byte[] {};
		}
		else 
		{
			return Reader.ReadBytes(__length);
		}
	}

	public void skipString() {
	
		uint __length = readInt();
		Reader.BaseStream.Seek(__length, SeekOrigin.Current);
	}
 
}
